#!/bin/env -S deno run -A

import { assert } from "https://deno.land/std@0.222.1/assert/assert.ts";
import { crayon } from "https://deno.land/x/crayon@3.3.3/mod.ts";
import { is_relevant } from "./lib/crypto.ts";
import { ApiClient } from "./lib/client.ts";
import { parallel, sum } from "./lib/utils.ts";
import { merge_request_parser, MergeRequest, note_parser } from "./lib/parser.ts";

async function main() {
	let { token } = parse_args(Deno.args);
	if (token === undefined) token = prompt_args();
	const client = new ApiClient("gitlab.cri.epita.fr", token);
	const merge_requests = await get_all_merge_requests(client);
	const total = sum(await parallel(merge_requests, (mr) => count_in_mr(mr, client)));
	console.log("Total", total, "oki");
}

function parse_args(args: string[]) {
	const [token_part] = args;
	const token = token_part as string | undefined;
	return { token };
}

function prompt_args() {
	const [bl, gr] = [crayon.blue, crayon.lightBlack];
	const message = `
${bl("Please enter a read-allowed gitlab token :")}
${gr("You can get one from https://gitlab.cri.epita.fr/-/user_settings/personal_access_tokens")}
${gr("with at least the permission 'read_api', format is :")}
${gr("> xxxxxxxxxxxxxxxxxxxx")}
>`.trim();
	const result = prompt(message);
	assert(result !== null);
	return result.trim();
}

async function get_all_merge_requests(client: ApiClient): Promise<MergeRequest[]> {
	return await client.fetch_parse_paginated(`api/v4/merge_requests`, merge_request_parser);
}

async function count_in_mr(mr: MergeRequest, client: ApiClient) {
	let total = 0;
	const mr_path = `api/v4/projects/${mr.project_id}/merge_requests/${mr.iid}`;
	const all_notes = await client.fetch_parse_paginated(`${mr_path}/notes`, note_parser);
	for (const c of all_notes) {
		if (!await is_relevant(c.author.username)) continue;
		if (c.body.toLowerCase().includes("oki")) total += 1;
	}
	return total;
}

if (import.meta.main) await main();
